<?php

/**
 * @file
 * Provides configuration UI for BitPay module.
 */

use Bitpay\Bitpay;
use Bitpay\PrivateKey;
use Bitpay\PublicKey;
use Bitpay\SinKey;

/**
 * Configuration options for BitPay.
 */
function bitpay_settings() {
  if (!class_exists('Bitpay\Bitpay')) {
    drupal_set_message('To configure BitPay module, please install bitpay/php-client via composer.', 'warning');
  }
  $form['bitpay_token_livenet'] = array(
    '#type' => 'item',
    '#title' => t('BitPay API access token'),
    '#description' => class_exists('Bitpay\Bitpay') ? (variable_get('bitpay_token_livenet') ? t('Configured') : t('Not configured')) : t('Not applicable because modern bitpay/php-client not installed via composer.'),
  );
  $form['bitpay_pairing_code_livenet'] = array(
    '#disabled' => !class_exists('Bitpay\Bitpay'),
    '#type' => 'textfield',
    '#title' => t('BitPay API pairing code'),
    '#description' => t('Visit your <a href="https://bitpay.com/dashboard/merchant/api-tokens">Manage API Tokens page</a>, click the "Add New Token" button, leave the "Require Authentication" checkbox checked, and enter the pairing code here.'),
    '#default_value' => variable_get('bitpay_pairing_code_livenet', ''),
  );
  $form['bitpay_token_testnet'] = array(
    '#type' => 'item',
    '#title' => t('Testnet BitPay API access token'),
    '#description' => class_exists('Bitpay\Bitpay') ? (variable_get('bitpay_token_testnet') ? t('Configured') : t('Not configured')) : t('Not applicable because modern bitpay/php-client not installed via composer.'),
  );
  $form['bitpay_pairing_code_testnet'] = array(
    '#disabled' => !class_exists('Bitpay\Bitpay'),
    '#type' => 'textfield',
    '#title' => t('Testnet BitPay API pairing code'),
    '#description' => t('Visit your <a href="https://test.bitpay.com/dashboard/merchant/api-tokens">Manage API Tokens page</a>, click the "Add New Token" button, leave the "Require Authentication" checkbox checked, and enter the pairing code here.'),
    '#default_value' => variable_get('bitpay_pairing_code_testnet', ''),
  );
  $form['bitpay_network'] = array(
    '#type' => 'radios',
    '#title' => t('Bitcoin network'),
    '#description' => t('Testnet uses the <a href="https://test.bitpay.com/">Bitpay test environment</a> and allows payment with free testnet coins.'),
    '#options' => array('livenet' => t('Livenet'), 'testnet' => t('Testnet')),
    '#default_value' => variable_get('bitpay_network', 'livenet'),
    '#disabled' => !class_exists('Bitpay\Bitpay'),
  );
  $form['bitpay_debug'] = array(
    '#type' => 'radios',
    '#title' => t('Debug logging'),
    '#description' => t('Enable to log BitPay API calls and invoice status updates.'),
    '#options' => array(t('Disabled'), t('Enabled')),
    '#default_value' => variable_get('bitpay_debug', FALSE),
    '#required' => TRUE,
  );
  $form['bitpay_notification_email'] = array(
    '#type' => 'textfield',
    '#title' => t('Notification e-mail'),
    '#description' => t('E-mail address where invoice update notifications should be sent.'),
    '#default_value' => variable_get('bitpay_notification_email', variable_get('site_mail', ini_get('sendmail_from'))),
    '#required' => TRUE,
  );
  $form['bitpay_redirect_url'] = array(
    '#type' => 'textfield',
    '#title' => t('Redirect path'),
    '#description' => t('The Drupal path where the customer should be redirected to after paying for the order; e.g. <em>donate/thanks</em>.'),
    '#default_value' => variable_get('bitpay_redirect_url', ''),
    '#required' => TRUE,
  );
  $form['bitpay_currency'] = array(
    '#type' => 'textfield',
    '#title' => t('Currency'),
    '#description' => t('The three-letter currency code - e.g. <em>USD</em> - in which the invoice was priced.'),
    '#default_value' => variable_get('bitpay_currency', 'BTC'),
    '#required' => TRUE,
  );
  $form['bitpay_physical'] = array(
    '#type' => 'radios',
    '#title' => t('Physical'),
    '#description' => t('Indicates whether anything is to be shipped with the order (if false, the buyer will be informed that nothing is to be shipped). This setting may be overridden.'),
    '#options' => array(t('False'), t('True')),
    '#default_value' => variable_get('bitpay_physical', TRUE),
    '#required' => TRUE,
  );
  $form['bitpay_transaction_speed'] = array(
    '#type' => 'select',
    '#title' => t('Transaction speed'),
    '#description' => t('See <a href="https://bitpay.com/downloads/bitpayApi.pdf">API docs</a> for more details.'),
    '#options' => array(
      'low' => t('Low'),
      'medium' => t('Medium'),
      'high' => t('High'),
    ),
    '#default_value' => variable_get('bitpay_transaction_speed', 'low'),
    '#required' => TRUE,
  );
  $form['#submit'][] = 'bitpay_settings_submit';
  return system_settings_form($form);
}

/**
 * Submit callback.
 */
function bitpay_settings_submit($form, &$form_state) {
  if (!class_exists('Bitpay\Bitpay')) {
    return;
  }
  global $base_url;
  foreach (array('livenet' => 'bitpay_pairing_code_livenet', 'testnet' => 'bitpay_pairing_code_testnet') as $network => $setting) {
    drupal_register_shutdown_function('variable_del', $setting);
    if (!$form_state['values'][$setting]) {
      continue;
    }
    variable_del("bitpay_token_$network");
    variable_del("bitpay_key_storage_password_$network");
    $password = drupal_random_key();
    $bitpay = new Bitpay(array(
      'bitpay' => array(
        'key_storage_password' => $password,
        'network' => $network,
        'private_key' => "private://bitpay_$network.key",
        'public_key' => "private://bitpay_$network.pub",
      ),
    ));
    // Generate and store private key.
    $keyManager = $bitpay->get('key_manager');
    $privateKey = new PrivateKey($bitpay->getContainer()->getParameter('bitpay.private_key'));
    $privateKey->generate();
    $keyManager->persist($privateKey);
    // Generate and store public key.
    $publicKey = new PublicKey($bitpay->getContainer()->getParameter('bitpay.public_key'));
    $publicKey->setPrivateKey($privateKey);
    $publicKey->generate();
    $keyManager->persist($publicKey);
    // Create access token.
    $sin = new SinKey();
    $sin->setPublicKey($publicKey);
    $sin->generate();
    $client = $bitpay->get('client');
    try {
      $token = $client->createToken(array(
        'id' => (string) $sin,
        'pairingCode' => $form_state['values'][$setting],
        'label' => $base_url,
      ));
    }
    catch (Exception $e) {
      drupal_set_message(t('There was a problem creating a !network token: %message', array('%message' => $e->getMessage(), '!network' => $network)), 'error');
    }
    if (!empty($token)) {
      variable_set("bitpay_token_$network", (string) $token);
      variable_set("bitpay_key_storage_password_$network", $password);
      drupal_set_message(t('New !network API token generated successfully. Encrypted keypair saved to the private filesystem.', array('!network' => $network)));
    }
    else {
      drupal_set_message(t('Failed to create !network API token. Bad pairing code?', array('!network' => $network)), 'warning');
    }
  }
}
